package cn.kdan.cloud.pdf.office.generate.service.impl;

import cn.kdan.cloud.pdf.office.generate.bo.ColumnInfoBO;
import cn.kdan.cloud.pdf.office.generate.bo.TemplateRenderingBO;
import cn.kdan.cloud.pdf.office.generate.config.GenerateCodeConfig;
import cn.kdan.cloud.pdf.office.generate.dto.GenerateCodeDTO;
import cn.kdan.cloud.pdf.office.generate.entity.TableColumn;
import cn.kdan.cloud.pdf.office.generate.service.GenerateCodeService;
import cn.kdan.cloud.pdf.office.generate.service.TableColumnsService;
import cn.kdan.cloud.pdf.office.generate.utils.ComPDFWordUtils;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.time.LocalDate;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author ZhouQiang 2022/7/24
 */
@Service
@Slf4j
public class GenerateCodeServiceImpl implements GenerateCodeService {

    private final FreeMarkerConfigurer freeMarkerConfigurer;
    private final TableColumnsService tableColumnsService;

    public static final String COLUMN_KEY = "PRI";

    public GenerateCodeServiceImpl(FreeMarkerConfigurer freeMarkerConfigurer, TableColumnsService tableColumnsService) {
        this.freeMarkerConfigurer = freeMarkerConfigurer;
        this.tableColumnsService = tableColumnsService;
    }

    @Override
    public byte[] generatorCode(GenerateCodeDTO generateCode) {

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (ZipOutputStream zip = new ZipOutputStream(baos)) {

            TemplateRenderingBO templateRendering = new TemplateRenderingBO();
            templateRendering.setAuthor(generateCode.getAuthor());
            templateRendering.setCreateDate(LocalDate.now().toString());
            templateRendering.setPackageName(StringUtils.isNotBlank(generateCode.getModuleName()) ? generateCode.getPackageName() + "." + generateCode.getModuleName() : generateCode.getPackageName());

            List<String> tableNames = generateCode.getTableNames();
            for (String tableName : tableNames) {
                String removePrefix = StringUtils.isNotBlank(generateCode.getTablePrefix()) ? tableName.replaceFirst(generateCode.getTablePrefix(), "") : tableName;
                String className = ComPDFWordUtils.convertClassName(removePrefix);
                templateRendering.setClassName(className);
                templateRendering.setTableName(tableName);
                templateRendering.setLowerClassName(className.substring(0, 1).toLowerCase() + className.substring(1));
                templateRendering.setPathName(ComPDFWordUtils.convertPathName(tableName));

                List<TableColumn> tableColumns = tableColumnsService.listTableColumnByName(tableName);
                List<ColumnInfoBO> columns = new LinkedList<>();
                for (TableColumn tableColumn : tableColumns) {
                    ColumnInfoBO column = new ColumnInfoBO();
                    String javaType = GenerateCodeConfig.getJavaType(tableColumn.getDataType());
                    if ("Date".equals(javaType)) {
                        templateRendering.setHasDate(true);
                    }
                    if ("BigDecimal".equals(javaType)) {
                        templateRendering.setHasBigDecimal(true);
                    }
                    column.setAttrName(ComPDFWordUtils.convertFieldName(tableColumn.getColumnName()));
                    column.setJavaType(javaType);
                    column.setHasPrimaryKey(COLUMN_KEY.equals(tableColumn.getColumnKey()));
                    column.setColumnName(tableColumn.getColumnName());
                    column.setColumnComment(tableColumn.getColumnComment());
                    columns.add(column);
                }
                templateRendering.setColumns(columns);

                this.generateTemplate(templateRendering, zip);
                templateRendering.setHasDate(false);
                templateRendering.setHasBigDecimal(false);
            }

            return baos.toByteArray();
        } catch (Exception e) {
            log.error("生成代码失败", e);
            throw new RuntimeException("生成代码失败");
        }
    }

    private void generateTemplate(TemplateRenderingBO templateRendering, ZipOutputStream zip) {
        List<String> templates = GenerateCodeConfig.listTemplate();
        for (String template : templates) {
            try {
                String content = FreeMarkerTemplateUtils.processTemplateIntoString(freeMarkerConfigurer.getConfiguration().getTemplate(template), templateRendering);
                ZipEntry zipEntry = new ZipEntry(getFileName(template, templateRendering.getPackageName(), templateRendering.getClassName()));
                zip.putNextEntry(zipEntry);
                IOUtils.write(content, zip, "UTF-8");
                zip.closeEntry();
            } catch (IOException | TemplateException e) {
                log.error("渲染模板失败");
                throw new RuntimeException("渲染模板失败");
            }
        }
    }

    public String getFileName(String templateName, String packageName, String className) {
        String packagePath = "main" + File.separator + "java" + File.separator + packageName.replace(".", File.separator) + File.separator;
        switch (templateName) {
            case "Controller.ftlh":
                return packagePath + "controller" + File.separator + "v1" + File.separator + className + "Controller.java";
            case "Entity.ftlh":
                return packagePath + "entity" + File.separator + className + ".java";
            case "Mapper.ftlh":
                return packagePath + "mapper" + File.separator + className + "Mapper.java";
            case "MapperXML.ftlh":
                return "main" + File.separator + "resources" + File.separator + "mapper" + File.separator + className + "Mapper.xml";
            case "Service.ftlh":
                return packagePath + "service" + File.separator + className + "Service.java";
            case "ServiceImpl.ftlh":
                return packagePath + "service" + File.separator + "impl" + File.separator + className + "ServiceImpl.java";
            default:
                return "empty";
        }
    }

}
