package cn.kdan.cloud.pdf.office.sso.service.impl;

import cn.kdan.cloud.pdf.office.api.account.vo.OauthClientDetails;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.sso.feign.UserFeignClient;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.sso.utils.TokenUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.security.oauth2.provider.endpoint.AuthorizationEndpoint;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.stereotype.Service;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.bind.support.SimpleSessionStatus;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.*;

@Service
@RefreshScope
public class AuthServiceImpl implements AuthService {

    private final Logger logger = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private RedisUtils<String,String> redisUtils;


    @Autowired
    private UserFeignClient userApi;

    @Autowired
    private AuthorizationEndpoint authorizationEndpoint;

    @Autowired
    private TokenEndpoint tokenEndpoint;



    @Value("${security.oauth2.client.user-authorization-uri}")
    private String authorizeUrl;

    @Value("${security.oauth2.client.access-token-uri}")
    private String tokenUri;


    @Override
    public String getAuthorizeCode(String userId,String clientId, HttpServletRequest request) {
        //获取用户的基本信息
        UserInfoVO userInfoVO = userApi.getById(userId).getResult();
        Principal principal = request.getUserPrincipal();
        Map<String, Object> model = new HashMap<>();
        Map<String,String> params = new HashMap<>();
        params.put("response_type","code");
        params.put("client_id",clientId);
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        SessionStatus status = new SimpleSessionStatus();
        ModelAndView authorizeMV = authorizationEndpoint.authorize(model, params, status, principal);
        RedirectView view = (RedirectView) authorizeMV.getView();
        if(view != null && StringUtils.isNotEmpty(view.getUrl())) {
            return view.getUrl().substring(view.getUrl().indexOf("code=")+5);
        } else {
            return null;
        }
    }

    @Override
    public TokenVO getToken(String code, String clientId, HttpServletRequest request) {
        Map<String,String> params = new HashMap<>();
        params.put("grant_type","authorization_code");
        params.put("code",code);
        params.put("client_id",clientId);
        //获取用户服务节点信息
        params.put("redirect_uri",AuthConstant.DEFAULT_REDIRECT_URI);
        params.put("client_secret","");
        User principal = new User(clientId,"", Collections.emptySet());
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        try {
            ResponseEntity<OAuth2AccessToken> oAuth2AccessTokenResponseEntity = tokenEndpoint.postAccessToken(authenticationToken, params);
            return convert(oAuth2AccessTokenResponseEntity.getBody());
        } catch (HttpRequestMethodNotSupportedException e) {
            logger.error(e.getMessage(),e);
        }
        return null;
    }

    @Override
    public TokenVO emailLogin(String email, String code, String appId, String platformType, String clientId, HttpServletRequest request) {
        //如果是管理平台就去查管理平台的用户表
        UserInfoVO userInfoVO = userApi.getByAppAccount(email,appId,platformType).getResult();
        Set<SimpleGrantedAuthority> authorities = new LinkedHashSet<>();
        String account = userInfoVO.getId();
        CustomUserDetails principal = new CustomUserDetails(userInfoVO.getId(),
                account,
                code,
                true,
                true,
                true,
                true,
                authorities);
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        OauthClientDetails clientDetails = userApi.getByClientId(clientId).getResult();
        if (clientDetails == null) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PLEASE_ADD_AUTH_CONFIG);
        }
        String authCode = TokenUtils.getAuthorizeCode(clientDetails.getClientId(), authenticationToken);
        OAuth2AccessToken token = TokenUtils.getToken(authCode, clientId, clientDetails.getClientSecret());
        return TokenUtils.convert(token);
    }

    private TokenVO convert(OAuth2AccessToken token){
        if(token == null) {
            return null;
        }
        TokenVO tokenVO = new TokenVO();
        tokenVO.setAccess_token(token.getValue());
        tokenVO.setExpires_in(token.getExpiresIn()+"");
        tokenVO.setToken_type(token.getTokenType());
        tokenVO.setRefresh_token(token.getRefreshToken().getValue());
        return tokenVO;
    }

    @Override
    public void refreshToken(Principal principal) {
        String token = ((OAuth2AuthenticationDetails)((OAuth2Authentication) principal).getDetails()).getTokenValue();
        Set<String> keySet = new HashSet<>();
        keySet.add(AuthConstant.ACCESS+token);
        keySet.add(AuthConstant.AUTH+token);
        keySet.add(AuthConstant.ACCESS_TO_REFRESH + token);
        long time = 30 * 60L;
        keySet.forEach(key -> redisUtils.expire(key, time));
    }

}
